@@ -13,7 +13,7 @@ require Exporter;
 require HTTP::Request;
 use Carp();
 
-$VERSION = "5.815";
+$VERSION = "5.817";
 
 my $CRLF = "\015\012";   # "\r\n" is not portable
 
@@ -56,7 +56,7 @@ sub POST
 	    my $boundary_index;
 	    for (my @tmp = @v; @tmp;) {
 		my($k, $v) = splice(@tmp, 0, 2);
-		if (lc($k) eq "boundary") {
+		if ($k eq "boundary") {
 		    $boundary = $v;
 		    $boundary_index = @v - @tmp - 1;
 		    last;
